#!/usr/bin/python
#
# Copyright 2024 Kaggle Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8
from __future__ import absolute_import

import logging
import six
import sys
import urllib3


class Configuration:

  def __init__(self):
    """Constructor."""
    # Default Base url
    self.host = _get_endpoint_from_env() or "https://www.kaggle.com/api/v1"

    # Authentication Settings
    # dict to store API key(s)
    self.api_key = {}
    # dict to store API prefix (e.g. Bearer)
    self.api_key_prefix = {}
    # Username for HTTP basic authentication
    self.username = ""
    # Password for HTTP basic authentication
    self.password = ""

    # Logging Settings
    self.logger = {
        "package_logger": logging.getLogger("kaggle"),
        "urllib3_logger": logging.getLogger("urllib3")
    }
    # Log format
    self.logger_format = '%(asctime)s %(levelname)s %(message)s'
    # Log stream handler
    self.logger_stream_handler = None
    # Log file handler
    self.logger_file_handler = None
    # Debug file location
    self.logger_file = None
    # Debug switch
    self.debug = False

  @property
  def logger_file(self):
    """The logger file.

    If the logger_file is None, then add stream handler and remove file
    handler. Otherwise, add file handler and remove stream handler.

    :param value: The logger_file path.
    :type: str
    """
    return self.__logger_file

  @logger_file.setter
  def logger_file(self, value):
    """The logger file.

    If the logger_file is None, then add stream handler and remove file
    handler. Otherwise, add file handler and remove stream handler.

    :param value: The logger_file path.
    :type: str
    """
    self.__logger_file = value
    if self.__logger_file:
      # If set logging file,
      # then add file handler and remove stream handler.
      self.logger_file_handler = logging.FileHandler(self.__logger_file)
      self.logger_file_handler.setFormatter(self.logger_formatter)
      for _, logger in six.iteritems(self.logger):
        logger.addHandler(self.logger_file_handler)
        if self.logger_stream_handler:
          logger.removeHandler(self.logger_stream_handler)
    else:
      # If not set logging file,
      # then add stream handler and remove file handler.
      self.logger_stream_handler = logging.StreamHandler()
      self.logger_stream_handler.setFormatter(self.logger_formatter)
      for _, logger in six.iteritems(self.logger):
        logger.addHandler(self.logger_stream_handler)
        if self.logger_file_handler:
          logger.removeHandler(self.logger_file_handler)

  @property
  def debug(self):
    """Debug status.

    :param value: The debug status, True or False.
    :type: bool
    """
    return self.__debug

  @debug.setter
  def debug(self, value):
    """Debug status.

    :param value: The debug status, True or False.
    :type: bool
    """
    self.__debug = value
    if self.__debug:
      # if debug status is True, turn on debug logging
      for _, logger in six.iteritems(self.logger):
        logger.setLevel(logging.DEBUG)
    else:
      # if debug status is False, turn off debug logging,
      # setting log level to default `logging.WARNING`
      for _, logger in six.iteritems(self.logger):
        logger.setLevel(logging.WARNING)

  @property
  def logger_format(self):
    """The logger format.

    The logger_formatter will be updated when sets logger_format.

    :param value: The format string.
    :type: str
    """
    return self.__logger_format

  @logger_format.setter
  def logger_format(self, value):
    """The logger format.

    The logger_formatter will be updated when sets logger_format.

    :param value: The format string.
    :type: str
    """
    self.__logger_format = value
    self.logger_formatter = logging.Formatter(self.__logger_format)

  def get_api_key_with_prefix(self, identifier):
    """Gets API key (with prefix if set).

    :param identifier: The identifier of apiKey.
    :return: The token for api key authentication.
    """
    if (self.api_key.get(identifier) and self.api_key_prefix.get(identifier)):
      return self.api_key_prefix[identifier] + ' ' + self.api_key[
          identifier]  # noqa: E501
    elif self.api_key.get(identifier):
      return self.api_key[identifier]

  def get_basic_auth_token(self):
    """Gets HTTP basic authentication header (string).

    :return: The token for basic HTTP authentication.
    """
    return urllib3.util.make_headers(basic_auth=self.username + ':' +
                                     self.password).get('authorization')

  def auth_settings(self):
    """Gets Auth Settings dict for api client.

    :return: The Auth Settings information dict.
    """
    return {
        'basicAuth': {
            'type': 'basic',
            'in': 'header',
            'key': 'Authorization',
            'value': self.get_basic_auth_token()
        },
    }

  def to_debug_report(self):
    """Gets the essential information for debugging.

    :return: The report for debugging.
    """
    return "Python SDK Debug Report:\n" \
           "OS: {env}\n" \
           "Python Version: {pyversion}\n" \
           "Version of the API: 1\n" \
           "SDK Package Version: 1". \
      format(env=sys.platform, pyversion=sys.version)


def _get_endpoint_from_env():
  import os
  endpoint = os.environ.get("KAGGLE_API_ENDPOINT")
  if endpoint is None:
    return None
  endpoint = endpoint.rstrip("/")
  if endpoint.endswith("/api/v1"):
    return endpoint
  return endpoint + "/api/v1"



#!/usr/bin/python
#
# Copyright 2024 Kaggle Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

import pprint
import re  # noqa: F401

import six


class StartBlobUploadRequest(object):
  """
    Attributes:
      project_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
  project_types = {
      'type': 'object',
      'name': 'str',
      'content_length': 'int',
      'content_type': 'str',
      'last_modified_epoch_seconds': 'int'
  }

  attribute_map = {
      'type': 'type',
      'name': 'name',
      'content_length': 'contentLength',
      'content_type': 'contentType',
      'last_modified_epoch_seconds': 'lastModifiedEpochSeconds'
  }

  def __init__(self,
               type=None,
               name=None,
               content_length=None,
               content_type=None,
               last_modified_epoch_seconds=None):  # noqa: E501
    """StartBlobUploadRequest - a model defined in Swagger"""  # noqa: E501

    self._type = None
    self._name = None
    self._content_length = None
    self._content_type = None
    self._last_modified_epoch_seconds = None
    self.discriminator = None

    if type is not None:
      self.type = type
    self.name = name
    self.content_length = content_length
    if content_type is not None:
      self.content_type = content_type
    if last_modified_epoch_seconds is not None:
      self.last_modified_epoch_seconds = last_modified_epoch_seconds

  @property
  def type(self):
    """Gets the type of this StartBlobUploadRequest.  # noqa: E501.

    The type of the blob (one of \"dataset\", \"model\", \"inbox\")  #
    noqa: E501

    :return: The type of this StartBlobUploadRequest.  # noqa: E501
    :rtype: object
    """
    return self._type

  @type.setter
  def type(self, type):
    """Sets the type of this StartBlobUploadRequest.

    The type of the blob (one of \"dataset\", \"model\", \"inbox\")  #
    noqa: E501

    :param type: The type of this StartBlobUploadRequest.  # noqa: E501
    :type: object
    """

    self._type = type

  @property
  def name(self):
    """Gets the name of this StartBlobUploadRequest.  # noqa: E501.

    Name of the file  # noqa: E501

    :return: The name of this StartBlobUploadRequest.  # noqa: E501
    :rtype: str
    """
    return self._name

  @name.setter
  def name(self, name):
    """Sets the name of this StartBlobUploadRequest.

    Name of the file  # noqa: E501

    :param name: The name of this StartBlobUploadRequest.  # noqa: E501
    :type: str
    """
    if name is None:
      raise ValueError(
          "Invalid value for `name`, must not be `None`")  # noqa: E501

    self._name = name

  @property
  def content_length(self):
    """Gets the content_length of this StartBlobUploadRequest.  # noqa: E501.

    Content length of the file in bytes  # noqa: E501

    :return: The content_length of this StartBlobUploadRequest. # noqa:
        E501
    :rtype: int
    """
    return self._content_length

  @content_length.setter
  def content_length(self, content_length):
    """Sets the content_length of this StartBlobUploadRequest.

    Content length of the file in bytes  # noqa: E501

    :param content_length: The content_length of this
        StartBlobUploadRequest. # noqa: E501
    :type: int
    """
    if content_length is None:
      raise ValueError("Invalid value for `content_length`, must not be `None`"
                      )  # noqa: E501

    self._content_length = content_length

  @property
  def content_type(self):
    """Gets the content_type of this StartBlobUploadRequest.  # noqa: E501.

    Content/MIME type (e.g. \"text/plain\") of the file  # noqa: E501

    :return: The content_type of this StartBlobUploadRequest. # noqa:
        E501
    :rtype: str
    """
    return self._content_type

  @content_type.setter
  def content_type(self, content_type):
    """Sets the content_type of this StartBlobUploadRequest.

    Content/MIME type (e.g. \"text/plain\") of the file  # noqa: E501

    :param content_type: The content_type of this
        StartBlobUploadRequest. # noqa: E501
    :type: str
    """

    self._content_type = content_type

  @property
  def last_modified_epoch_seconds(self):
    """Gets the last_modified_epoch_seconds of this StartBlobUploadRequest.  #
    noqa: E501.

    Last modified date of file in seconds since epoch in UTC  # noqa:
    E501

    :return: The last_modified_epoch_seconds of this
        StartBlobUploadRequest. # noqa: E501
    :rtype: int
    """
    return self._last_modified_epoch_seconds

  @last_modified_epoch_seconds.setter
  def last_modified_epoch_seconds(self, last_modified_epoch_seconds):
    """Sets the last_modified_epoch_seconds of this StartBlobUploadRequest.

    Last modified date of file in seconds since epoch in UTC  # noqa:
    E501

    :param last_modified_epoch_seconds: The last_modified_epoch_seconds
        of this StartBlobUploadRequest. # noqa: E501
    :type: int
    """

    self._last_modified_epoch_seconds = last_modified_epoch_seconds

  def to_dict(self):
    """Returns the model properties as a dict."""
    result = {}

    for attr, _ in six.iteritems(self.project_types):
      value = getattr(self, attr)
      if isinstance(value, list):
        result[attr] = list(
            map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
      elif hasattr(value, "to_dict"):
        result[attr] = value.to_dict()
      elif isinstance(value, dict):
        result[attr] = dict(
            map(
                lambda item: (item[0], item[1].to_dict())
                if hasattr(item[1], "to_dict") else item, value.items()))
      else:
        result[attr] = value

    return result

  def to_str(self):
    """Returns the string representation of the model."""
    return pprint.pformat(self.to_dict())

  def __repr__(self):
    """For `print` and `pprint`"""
    return self.to_str()

  def __eq__(self, other):
    """Returns true if both objects are equal."""
    if not isinstance(other, StartBlobUploadRequest):
      return False

    return self.__dict__ == other.__dict__

  def __ne__(self, other):
    """Returns true if both objects are not equal."""
    return not self == other



import os
from enum import Enum


class KaggleEnv(Enum):
  LOCAL = 0  # localhost
  STAGING = 1  # staging.kaggle.com
  ADMIN = 2  # admin.kaggle.com
  QA = 3  # qa.kaggle.com
  # Direct prod access is not allowed to have IAP protection during testing, but we support basic auth.
  PROD = 4  # www.kaggle.com


_env_to_endpoint = {
    KaggleEnv.LOCAL: 'http://localhost',
    KaggleEnv.STAGING: 'https://staging.kaggle.com',
    KaggleEnv.ADMIN: 'https://admin.kaggle.com',
    KaggleEnv.QA: 'https://qa.kaggle.com',
    # See the comment above in KaggleEnv enum.
    KaggleEnv.PROD: 'https://www.kaggle.com',
}


def get_endpoint(env: KaggleEnv):
  return _env_to_endpoint[env]


def get_env():
  env = os.getenv('KAGGLE_API_ENVIRONMENT')
  if env is None or env == 'PROD':
    return KaggleEnv.PROD
  if env == 'LOCALHOST':
    return KaggleEnv.LOCAL
  if env == 'ADMIN':
    return KaggleEnv.ADMIN
  if env == 'STAGING':
    return KaggleEnv.STAGING
  if env == 'QA':
    return KaggleEnv.QA
  raise Exception(f'Unrecognized value in KAGGLE_API_ENVIRONMENT: "{env}"')



#!/usr/bin/python
#
# Copyright 2024 Kaggle Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

import pprint
import re  # noqa: F401

import six


class DatasetColumn(object):
  """
    Attributes:
      column_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
  column_types = {
      'order': 'float',
      'name': 'str',
      'type': 'str',
      'original_type': 'str',
      'description': 'str'
  }

  attribute_map = {
      'order': 'order',
      'name': 'name',
      'type': 'type',
      'original_type': 'originalType',
      'description': 'description'
  }

  def __init__(self,
               order=None,
               name=None,
               type=None,
               original_type=None,
               description=None):  # noqa: E501
    """DatasetColumn - a model defined in Swagger"""  # noqa: E501

    self._order = None
    self._name = None
    self._type = None
    self._original_type = None
    self._description = None
    self.discriminator = None

    if order is not None:
      self.order = order
    if name is not None:
      self.name = name
    if type is not None:
      self.type = type
    if original_type is not None:
      self.original_type = original_type
    if description is not None:
      self.description = description

  @property
  def order(self):
    """Gets the order of this DatasetColumn.  # noqa: E501.

    The order that the column comes in, 0-based. (The first column is 0,
    second is 1, etc.)  # noqa: E501

    :return: The order of this DatasetColumn.  # noqa: E501
    :rtype: float
    """
    return self._order

  @order.setter
  def order(self, order):
    """Sets the order of this DatasetColumn.

    The order that the column comes in, 0-based. (The first column is 0,
    second is 1, etc.)  # noqa: E501

    :param order: The order of this DatasetColumn.  # noqa: E501
    :type: float
    """

    self._order = order

  @property
  def name(self):
    """Gets the name of this DatasetColumn.  # noqa: E501.

    The column name  # noqa: E501

    :return: The name of this DatasetColumn.  # noqa: E501
    :rtype: str
    """
    return self._name

  @name.setter
  def name(self, name):
    """Sets the name of this DatasetColumn.

    The column name  # noqa: E501

    :param name: The name of this DatasetColumn.  # noqa: E501
    :type: str
    """

    self._name = name

  @property
  def type(self):
    """Gets the type of this DatasetColumn.  # noqa: E501.

    The type of all of the fields in the column. Please see the data
    types on
    https://github.com/Kaggle/kaggle-api/wiki/Dataset-Metadata
     # noqa: E501

    :return: The type of this DatasetColumn.  # noqa: E501
    :rtype: str
    """
    return self._type

  @type.setter
  def type(self, type):
    """Sets the type of this DatasetColumn.

    The type of all of the fields in the column. Please see the data
    types on
    https://github.com/Kaggle/kaggle-api/wiki/Dataset-Metadata
     # noqa: E501

    :param type: The type of this DatasetColumn.  # noqa: E501
    :type: str
    """

    self._type = type

  @property
  def original_type(self):
    """Gets the original_type of this DatasetColumn.  # noqa: E501.

    Used to store the original type of the column, which will be converted to Kaggle's types. For example, an `originalType` of `\"integer\"` would convert to a `type` of `\"numeric\"`  # noqa: E501

    :return: The original_type of this DatasetColumn.  # noqa: E501
    :rtype: str
    """
    return self._original_type

  @original_type.setter
  def original_type(self, original_type):
    """Sets the original_type of this DatasetColumn.

    Used to store the original type of the column, which will be converted to Kaggle's types. For example, an `originalType` of `\"integer\"` would convert to a `type` of `\"numeric\"`  # noqa: E501

    :param original_type: The original_type of this DatasetColumn.  # noqa: E501
    :type: str
    """

    self._original_type = original_type

  @property
  def description(self):
    """Gets the description of this DatasetColumn.  # noqa: E501.

    The description of the column  # noqa: E501

    :return: The description of this DatasetColumn.  # noqa: E501
    :rtype: str
    """
    return self._description

  @description.setter
  def description(self, description):
    """Sets the description of this DatasetColumn.

    The description of the column  # noqa: E501

    :param description: The description of this DatasetColumn. # noqa:
        E501
    :type: str
    """

    self._description = description

  def to_dict(self):
    """Returns the model properties as a dict."""
    result = {}

    for attr, _ in six.iteritems(self.column_types):
      value = getattr(self, attr)
      if isinstance(value, list):
        result[attr] = list(
            map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
      elif hasattr(value, "to_dict"):
        result[attr] = value.to_dict()
      elif isinstance(value, dict):
        result[attr] = dict(
            map(
                lambda item: (item[0], item[1].to_dict())
                if hasattr(item[1], "to_dict") else item, value.items()))
      else:
        result[attr] = value

    return result

  def to_str(self):
    """Returns the string representation of the model."""
    return pprint.pformat(self.to_dict())

  def __repr__(self):
    """For `print` and `pprint`"""
    return self.to_str()

  def __eq__(self, other):
    """Returns true if both objects are equal."""
    if not isinstance(other, DatasetColumn):
      return False

    return self.__dict__ == other.__dict__

  def __ne__(self, other):
    """Returns true if both objects are not equal."""
    return not self == other



#!/usr/bin/python
#
# Copyright 2024 Kaggle Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

import pprint
import re  # noqa: F401

import six

from kaggle.models.dataset_column import DatasetColumn  # noqa: F401,E501


class UploadFile(object):
  """
    Attributes:
      column_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
  column_types = {
      'token': 'str',
      'description': 'str',
      'columns': 'list[DatasetColumn]'
  }

  attribute_map = {
      'token': 'token',
      'description': 'description',
      'columns': 'columns'
  }

  def __init__(self, token=None, description=None, columns=None):  # noqa: E501
    """UploadFile - a model defined in Swagger"""  # noqa: E501

    self._token = None
    self._description = None
    self._columns = None
    self.discriminator = None

    if token is not None:
      self.token = token
    if description is not None:
      self.description = description
    if columns is not None:
      self.columns = columns

  @property
  def token(self):
    """Gets the token of this UploadFile.  # noqa: E501.

    A token referencing a specific file upload that can be used across
    requests  # noqa: E501

    :return: The token of this UploadFile.  # noqa: E501
    :rtype: str
    """
    return self._token

  @token.setter
  def token(self, token):
    """Sets the token of this UploadFile.

    A token referencing a specific file upload that can be used across
    requests  # noqa: E501

    :param token: The token of this UploadFile.  # noqa: E501
    :type: str
    """

    self._token = token

  @property
  def description(self):
    """Gets the description of this UploadFile.  # noqa: E501.

    The file description  # noqa: E501

    :return: The description of this UploadFile.  # noqa: E501
    :rtype: str
    """
    return self._description

  @description.setter
  def description(self, description):
    """Sets the description of this UploadFile.

    The file description  # noqa: E501

    :param description: The description of this UploadFile. # noqa: E501
    :type: str
    """

    self._description = description

  @property
  def columns(self):
    """Gets the columns of this UploadFile.  # noqa: E501.

    A list of dataset column metadata  # noqa: E501

    :return: The columns of this UploadFile.  # noqa: E501
    :rtype: list[DatasetColumn]
    """
    return self._columns

  @columns.setter
  def columns(self, columns):
    """Sets the columns of this UploadFile.

    A list of dataset column metadata  # noqa: E501

    :param columns: The columns of this UploadFile.  # noqa: E501
    :type: list[DatasetColumn]
    """

    self._columns = columns

  def to_dict(self):
    """Returns the model properties as a dict."""
    result = {}

    for attr, _ in six.iteritems(self.column_types):
      value = getattr(self, attr)
      if isinstance(value, list):
        result[attr] = list(
            map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value))
      elif hasattr(value, "to_dict"):
        result[attr] = value.to_dict()
      elif isinstance(value, dict):
        result[attr] = dict(
            map(
                lambda item: (item[0], item[1].to_dict())
                if hasattr(item[1], "to_dict") else item, value.items()))
      else:
        result[attr] = value

    return result

  def to_str(self):
    """Returns the string representation of the model."""
    return pprint.pformat(self.to_dict())

  def __repr__(self):
    """For `print` and `pprint`"""
    return self.to_str()

  def __eq__(self, other):
    """Returns true if both objects are equal."""
    if not isinstance(other, UploadFile):
      return False

    return self.__dict__ == other.__dict__

  def __ne__(self, other):
    """Returns true if both objects are not equal."""
    return not self == other



from kaggle.api.kaggle_api_extended import KaggleApi

# python -m unittest tests.test_authenticate

import os
import unittest


class TestAuthenticate(unittest.TestCase):

  def setUp(self):
    print("setup             class:%s" % self)

  def tearDown(self):
    print("teardown          class:TestStuff")

  # Environment

  def test_environment_variables(self):
    os.environ['KAGGLE_USERNAME'] = 'dinosaur'
    os.environ['KAGGLE_KEY'] = 'xxxxxxxxxxxx'
    api = KaggleApi()

    # We haven't authenticated yet
    self.assertTrue("key" not in api.config_values)
    self.assertTrue("username" not in api.config_values)
    api.authenticate()

    # Should be set from the environment
    self.assertEqual(api.config_values['key'], 'xxxxxxxxxxxx')
    self.assertEqual(api.config_values['username'], 'dinosaur')

  # Configuration Actions

  def test_config_actions(self):
    api = KaggleApi()

    self.assertTrue(api.config_dir.endswith('kaggle'))
    self.assertEqual(api.get_config_value('doesntexist'), None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
#
# Copyright 2019 Kaggle Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding=utf-8
from datetime import datetime


class Competition(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)
    self.tags = [Tag(t) for t in self.tags]

  def __repr__(self):
    return self.ref


class SubmitResult(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return self.message


class Submission(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)
    if self.totalBytes is None:
      self.size = None
    else:
      self.size = File.get_size(self.totalBytes)

  def __repr__(self):
    return str(self.ref)


class LeaderboardEntry(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return self.teamId


class Dataset(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)
    self.tags = [Tag(t) for t in self.tags]
    self.files = [File(f) for f in self.files]
    self.versions = [DatasetVersion(v) for v in self.versions]
    self.size = File.get_size(self.totalBytes)

  def __repr__(self):
    return self.ref


class Model(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return self.ref


class Metadata(object):

  def __init__(self, init_info):
    parsed_info = {k: parse(v) for k, v in init_info.items()}
    # backwards compatibility
    self.id = parsed_info["ownerUser"] + "/" + parsed_info['datasetSlug']
    self.id_no = parsed_info['datasetId']
    self.__dict__.update(parsed_info)

  def __repr__(self):
    return str(self.datasetId)


class DatasetVersion(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return str(self.versionNumber)


class File(object):

  def __init__(self, init_dict):
    try:  # TODO Remove try-block
      parsed_dict = {k: parse(v) for k, v in init_dict.items()}
      self.__dict__.update(parsed_dict)
      self.size = File.get_size(self.totalBytes)
    except AttributeError:
      self.name = init_dict.name
      self.creation_date = init_dict.creation_date
      try:
        self.size = File.get_size(init_dict.total_bytes)
      except Exception: # AttributeError would be preferred but doesn't work.
        self.size = File.get_size(init_dict.size)

  def __repr__(self):
    return self.name

  @staticmethod
  def get_size(size, precision=0):
    suffixes = ['B', 'KB', 'MB', 'GB', 'TB']
    suffix_index = 0
    while size >= 1024 and suffix_index < 4:
      suffix_index += 1
      size /= 1024.0
    return '%.*f%s' % (precision, size, suffixes[suffix_index])


class Tag(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return self.ref


class DatasetNewVersionResponse(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return self.url


class DatasetNewResponse(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return self.url


class ListFilesResult(object):

  def __init__(self, init_dict):
    try:  # TODO Remove try-block
      self.error_message = init_dict['errorMessage']
      files = init_dict['datasetFiles']
      token = init_dict['nextPageToken']
    except TypeError:
      self.error_message = init_dict.error_message
      files = init_dict.dataset_files
      token = init_dict.next_page_token
    if files:
      self.files = [File(f) for f in files]
    else:
      self.files = {}
    if token:
      self.nextPageToken = token
    else:
      self.nextPageToken = ""

  def __repr__(self):
    return self.error_message


class Kernel:

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return self.title


class KernelPushResponse(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return self.newUrl


class ModelNewResponse(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return self.url


class ModelDeleteResponse(object):

  def __init__(self, init_dict):
    parsed_dict = {k: parse(v) for k, v in init_dict.items()}
    self.__dict__.update(parsed_dict)

  def __repr__(self):
    return self.error


def parse(string):
  time_formats = [
      '%Y-%m-%dT%H:%M:%S', '%Y-%m-%dT%H:%M:%SZ', '%Y-%m-%dT%H:%M:%S.%f',
      '%Y-%m-%dT%H:%M:%S.%fZ'
  ]
  for t in time_formats:
    try:
      result = datetime.strptime(string[:26], t).replace(microsecond=0)
      return result
    except:
      pass
  return string


class ResumableUploadResult(object):
  # Upload was complete, i.e., all bytes were received by the server.
  COMPLETE = 1

  # There was a non-transient error during the upload or the upload expired.
  # The upload cannot be resumed so it should be restarted from scratch
  # (i.e., call /api/v1/files/upload to initiate the upload and get the
  # create/upload url and token).
  FAILED = 2

  # Upload was interrupted due to some (transient) failure but it can be
  # safely resumed.
  INCOMPLETE = 3

  def __init__(self, result, bytes_uploaded=None):
    self.result = result
    self.bytes_uploaded = bytes_uploaded
    self.start_at = 0 if bytes_uploaded is None else bytes_uploaded + 1

  @staticmethod
  def Complete():
    return ResumableUploadResult(ResumableUploadResult.COMPLETE)

  @staticmethod
  def Failed():
    return ResumableUploadResult(ResumableUploadResult.FAILED)

  @staticmethod
  def Incomplete(bytes_uploaded=None):
    return ResumableUploadResult(ResumableUploadResult.INCOMPLETE,
                                 bytes_uploaded)



import os
import unittest
from typing import List

from kaggle.api.kaggle_api_extended import KaggleApi

MODEL_HANDLE = "keras/bert"
MODEL_ID = 2819

# TODO(messick) Add a test that creates a dataset w/o specifying privacy that is created private.

class TestModels(unittest.TestCase):
    def setUp(self):
        self.api = KaggleApi()
        self.api.authenticate()

    def test_list_models(self) -> None:
        models = self.api.model_list()
        self.assertGreater(len(models), 0)

    def test_get_model(self) -> None:
        model = self.api.model_get(MODEL_HANDLE)
        self.assertEqual(MODEL_ID, model.id)
